/*
 * key_debounce.c
 *
 *  Created on: Jan 11, 2010
 *      Author: jgnoss
 *      based on Peter Danneggers debounce
 *      http://www.mikrocontroller.net
 */


#include "key_debounce.h"


volatile uint8_t key_state; // debounced and inverted key state:
volatile uint8_t key_press; // key press detect
volatile uint8_t key_rpt; // key long press and repeat



void init_debounce(void){

	KEY_DDR	&= ~(ALL_KEYS);
	KEY_GPIO |= (ALL_KEYS);
}

///////////////////////////////////////////////////////////////////
//
// check if a key has been pressed. Each pressed key is reported
// only once
//
uint8_t get_key_press(uint8_t key_mask) {
//	cli();
	// read and clear atomic !
	key_mask &= key_press; // read key(s)
	key_press ^= key_mask; // clear key(s)
//	sei();
	return key_mask;
}

///////////////////////////////////////////////////////////////////
//
// check if a key has been pressed long enough such that the
// key repeat functionality kicks in. After a small setup delay
// the key is reported beeing pressed in subsequent calls
// to this function. This simulates the user repeatedly
// pressing and releasing the key.
//
uint8_t get_key_rpt(uint8_t key_mask) {
//	cli();
	// read and clear atomic !
	key_mask &= key_rpt; // read key(s)
	key_rpt ^= key_mask; // clear key(s)
//	sei();
	return key_mask;
}

///////////////////////////////////////////////////////////////////
//
uint8_t get_key_short(uint8_t key_mask) {
//	cli();
	// read key state and key press atomic !
	return get_key_press(~key_state & key_mask);
}

///////////////////////////////////////////////////////////////////
//
uint8_t get_key_long(uint8_t key_mask) {
	return get_key_press(get_key_rpt(key_mask));
}


int8_t getKey(void) {

	if (get_key_short(Key_Right)) {
		return Key_Right;
	}

	if (get_key_short(Key_Left)) {
		return Key_Left;
	}

	if (get_key_short(Key_Cent)) {
		return Key_Cent;
	}

	if (get_key_short(Key_Up)) {
		return Key_Up;
	}

	if (get_key_short(Key_Down)) {
		return Key_Down;
	}

	if (get_key_rpt(Key_Up)) {
		return Key_LUp;
	}

	if (get_key_rpt(Key_Down)) {
		return Key_LDown;
	}

	return KEY_NULL;
}




