/*
 * key_debounce.h
 *
 *  Created on: Jan 11, 2010
 *      Author: jgnoss
 *      based on Peter Danneggers debounce
 *      http://www.mikrocontroller.net
 */

#ifndef KEY_DEBOUNCE_H_
#define KEY_DEBOUNCE_H_


#include <avr/io.h>

#define 	KEY_DDR			DDRD
#define 	KEY_GPIO        PORTD
#define 	KEY_PIN         PIND

#define 	Key_Down		(1 << 3)
#define 	Key_Right		(1 << 4)
#define 	Key_Cent		(1 << 5)
#define 	Key_Left		(1 << 6)
#define 	Key_Up			(1 << 7)


#define 	ALL_KEYS        (Key_Left | Key_Right | Key_Up | Key_Down | Key_Cent )
#define 	REPEAT_MASK     (Key_Up | Key_Down)

#define 	REPEAT_START    50
#define 	REPEAT_NEXT     20
#define 	KEY_NULL		-1

#define 	Key_LUp		Key_Up+10
#define 	Key_LDown	Key_Down+10


extern volatile uint8_t key_state;
extern volatile uint8_t key_press;
extern volatile uint8_t key_rpt;


void init_debounce(void);


int8_t getKey(void);

uint8_t get_key_press(uint8_t key_mask);
uint8_t get_key_rpt(uint8_t key_mask);
uint8_t get_key_short(uint8_t key_mask);
uint8_t get_key_long(uint8_t key_mask);


static inline void keyDecode_CallBack(void){
	static uint8_t ct0, ct1, rpt;
	uint8_t k = 0;

	k = key_state ^ ~KEY_PIN; // key changed ?
	ct0 = ~(ct0 & k); // reset or count ct06
	ct1 = ct0 ^ (ct1 & k); // reset or count ct1l
	k &= ct0 & ct1; // count until roll over
	key_state ^= k; // then toggle debounced state
	key_press |= key_state & k; // 0->1: key press detect

	if ((key_state & REPEAT_MASK) == 0) // check repeat function
		rpt = REPEAT_START; // start delay6
	if (--rpt == 0) {
		rpt = REPEAT_NEXT; // repeat delay
		key_rpt |= key_state & REPEAT_MASK;
	}

}

#endif /* KEY_DEBOUNCE_H_ */
