/**
 * @file
 * @author  Juergen Gnoss <jgnoss@hotmail.com>
 * @version 0.0.0.01
 *
 * @section LICENSE
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details at
 * http://www.gnu.org/copyleft/gpl.html
 *
 * @section DESCRIPTION
 *
 *
 */

#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>
#include <util/delay.h>
#include "p_Globals.h"
#include "spmc_LCD_Menu.h"
#include "key_debounce.h"
#include "lcd.h"
#include "printf.h"
#include "actuators.h"


#ifndef F_CPU
#define F_CPU 16000000UL
#endif


ilum_Type_t actlamps = lt_Top;

volatile systemData_t systemData = {
		FALSE,		//updateLCD;
		FALSE,		//call_UpdateLCD;
		FALSE,		//isCounting
		lt_Bottom,	//actLamp
		0 ,			//actSec;
		0 ,			//progSec;
		0 			//sysSec;
};

ISR( TIMER1_COMPA_vect) {
	static uint16_t t1Retarder = 0;

	t1Retarder++;

	if (t1Retarder == 120) {
		t1Retarder = 0;
		systemData.sysSec++;
		if (systemData.isCounting){
			switch (systemData.actLamp) {
			case lt_Bottom:
				Lamp_BOT_ON();
				break;
			case lt_Top:
				Lamp_TOP_ON();
				break;
			case lt_Both:
				Lamp_TOP_ON();
				Lamp_BOT_ON();
				break;
			}
			if (systemData.actSec) {
				systemData.actSec--;
			}
		} else {
			Lamp_TOP_OFF();
			Lamp_BOT_OFF();
		}
	}
}

ISR( TIMER2_OVF_vect) {
	static uint16_t retarder = 0;

	retarder++;

	lcd_BL_PWM_CallBack();

	if (retarder == 62) {
		retarder = 0;
		keyDecode_CallBack();
	}
}

void longDelay(uint16_t l);


void initTimers(void) {

	TCCR1B  |= ((1 << WGM12) | (1 << CS12) | (1 << CS10));
	TIMSK |= (1 << OCIE1A);
	OCR1A = 128;

	TCCR2 |= (1 << CS20);
	TIMSK |= (1 << TOIE2);
}


int main(void) {

	init_LCDMenu();
	initTimers();
	init_Actuators();
	sei();

	for (;;) { /* loop forever */

		handle_LCD_Menu(&menuParameter);
	}
}

/******************************************************************************************/

void longDelay(uint16_t l){
	uint16_t cn = l;
	while (cn--) {
		_delay_ms(1);
	}
}
