/**
 * @file spmc_LCD_Menu.c
 * @author jgnoss
 * @date Sep 23, 2012
 *
 * based on the Menu State machine from AVR Butterfly
 *
 */

#include <stdio.h>
#include <inttypes.h>
#include <avr/pgmspace.h>
#include "version.h"
#include "p_Globals.h"
#include "printf.h"
#include "key_debounce.h"
#include "lcd.h"
#include "spmc_LCD_Menu.h"

char lampString[3][5] = { {'b','o','t',0}, {'t','o','p',0}, {'b','o','t','h',0}};
uint8_t timeCurPos[5] = {6, 8,9,11,12};

#define DBG_LEVEL_KEY 1
#define DBG_LEVEL_MENU 2
//#define DEBUGLEVEL  ((1 << DBG_LEVEL_KEY) |  (1 << DBG_LEVEL_MENU))

uint8_t pTime_Digits[5] = { 0, 0, 0, 0, 0};
uint8_t s_Digits[5] = { 0, 0, 0, 0, 0};
char dspLines[4][20] = { "", "", "", "" };

menuParameter_t menuParameter;


const char MT_Presentation[] = 	"WiLumi          ";
const char MT_Welcome[] = 			"Welcome         ";
const char MT_Lamp_Select[] = 		"Select Lamp     ";
const char MT_Time_Select[] = 		"Select Time     ";
const char MT_InitWithOK[] = 		"Init with OK    ";
const char MT_ClearCounter[] = 	"Reset Time      ";


const char SST_Not_Yet[] = "Not yet             ";

const MENU_NEXTSTATE_t menu_nextstate[] = {
//  	STATE 				INPUT			NEXT STATE


	{ ST_MP_Welcome,		Key_Right, 		ST_MP_Lamp_Select },
	{ ST_MP_Welcome, 		Key_Down, 		ST_Show_Rev },

	{ ST_MP_Lamp_Select,	Key_Right, 		ST_MP_Time_Select },
	{ ST_MP_Lamp_Select,	Key_Cent, 		ST_Lamp_Select },
	{ ST_MP_Lamp_Select,	Key_Left, 		ST_MP_Welcome },

	{ ST_MP_Time_Select,	Key_Cent, 		ST_Time_Select },
	{ ST_MP_Time_Select,	Key_Right, 		ST_Init_W_OK },
	{ ST_MP_Time_Select,	Key_Left, 		ST_MP_Lamp_Select },


	{ ST_NotYet, 			Key_Left, 		ST_MP_Welcome },
	{ ST_NotYet, 			Key_Right, 		ST_MP_Welcome },
	{ ST_NotYet, 			Key_Up, 		ST_MP_Welcome },
	{ ST_NotYet, 			Key_Down, 		ST_MP_Welcome },
	{ ST_NotYet, 			Key_Cent, 		ST_MP_Welcome },

		{ 0, 0, 0 }, };

const MENU_STATE_t menu_state[] = {
//  	STATE					STATE TEXT		STATE_FUNC
		{ ST_MP_Welcome, 		MT_Welcome, 	NULL },
		{ ST_MP_Lamp_Select, 	MT_Lamp_Select, NULL },
		{ ST_MP_Time_Select,	MT_Time_Select, NULL },

		{ ST_Lamp_Select, 		NULL, 			on_ST_Lamp_Select },
		{ ST_Time_Select, 		NULL, 			on_ST_Time_Select },

		{ ST_Init_W_OK, 		NULL, 			on_ST_Init_W_OK },

		{ ST_ResetTimer, 		NULL, 			on_ST_ResetTimer },

		{ ST_NotYet, 			SST_Not_Yet,	 NULL },
		{ 0, 0, 0 } };

/***************************************************************/

void handle_LCD_Menu(menuParameter_t *menuParam) {
	int8_t i = 0, j = 0;
	int8_t input = KEY_NULL;

	#if DEBUGLEVEL
	static char pcLineText[128];
#endif

	spmc_Update_LCD();

	if (menuParam->statetext) {
		spmc_WriteMenuText(menuParam->statetext);
		menuParam->statetext = NULL;
	}

	input = getKey();


#if DEBUGLEVEL & (1 << DBG_LEVEL_KEY)
	if (input != KEY_NULL) {
		tfp_sprintf(pcLineText, "\r\nKey : %u\r\n", input);
		uart1_puts(pcLineText);
//			spmc_WriteMenuText("WriteMenuText");


	}
#endif
	if (menuParam->pStateFunc) {
		menuParam->lcdM_NextState = menuParam->pStateFunc(input);
	} else {
		if (input != KEY_NULL) {
			menuParam->lcdM_NextState = lcdMenuStateMachine(
					menuParam->lcdM_State, input);
		}
	}

	if (menuParam->lcdM_NextState != menuParam->lcdM_State) {
#if DEBUGLEVEL & (1 << DBG_LEVEL_MENU)
		tfp_sprintf(pcLineText, "\r\n state %u  got : %u\r\n",
				menuParam->lcdM_State, menuParam->lcdM_NextState);
		uart1_puts(pcLineText);
#endif
		menuParam->lcdM_PrevState = menuParam->lcdM_State;
		menuParam->lcdM_State = menuParam->lcdM_NextState;
		for (i = 0; (j = menu_state[i].state); i++) {
			if (j == menuParam->lcdM_State) {
				menuParam->statetext = menu_state[i].pText;
				menuParam->pStateFunc = menu_state[i].pFunc;

#if DEBUGLEVEL & (1 << DBG_LEVEL_MENU)
				if (menuParam->statetext) {
					tfp_sprintf(pcLineText, "\r\n found text %s \r\n",
							menuParam->statetext);
				} else {
					if (menuParam->pStateFunc) {
						tfp_sprintf(pcLineText, "\r\n found func \r\n");
					}
				}
				uart1_puts(pcLineText);
#endif
				break;
			} else {
#if DEBUGLEVEL & (1 << DBG_LEVEL_MENU)
				tfp_sprintf(pcLineText, ".");
				uart1_puts(pcLineText);
#endif
			}
		}
	}
}

/****************************************************************/
void changeToState(uint8_t state, uint8_t saveState) {
	if (saveState) {
		menuParameter.lcd_SavedState = menuParameter.lcdM_State;
	}
	menuParameter.enter = 1;
	menuParameter.lcdM_NextState = state;
}

uint8_t lcdMenuStateMachine(uint8_t state, int8_t stimuli) {
	uint8_t nextstate = state; // Default stay in same state
	uint8_t i, j;

	for (i = 0; (j = menu_nextstate[i].state); i++) {
		if (j == state && menu_nextstate[i].input == stimuli) {
			nextstate = menu_nextstate[i].nextstate;
			break;
		}
	}
	return nextstate;
}

void init_LCDMenu(void) {

	init_debounce();
	lcd_init(LCD_DISP_ON);

	lcd_puts("Hello");
	menuParameter.lcdM_State = ST_MP_Welcome;
	menuParameter.lcdM_PrevState = ST_MP_Welcome;
	menuParameter.lcdM_NextState = ST_MP_Welcome;
	menuParameter.statetext = MT_Welcome;
	menuParameter.pStateFunc = NULL;
	menuParameter.enter = 1;
}

uint8_t on_ST_Show_Rev(int8_t input) {
	uint8_t retval = ST_Show_Rev;

	if (menuParameter.enter) {
		menuParameter.enter = 0;
		tfp_sprintf((char*) dspLines[2], "Version %s", gVersion);
		spmc_WriteMenuText(MT_Presentation);
		spmc_WriteToLine(2, dspLines[2]);
	} else {
		if (input == Key_Left ) {
			menuParameter.enter = 1;
			retval = ST_MP_Welcome;
		}
	}
	return retval;
}

uint8_t on_ST_EStop(int8_t input) {
	static uint8_t retval = ST_EStop;

	if (menuParameter.enter) {
		menuParameter.enter = 0;
		spmc_WriteMenuText("MP XX");
		retval = ST_EStop;
	}
	switch ((uint8_t) input) {
	case Key_Left :
		menuParameter.enter = 1;
		retval = ST_MP_Welcome;
		break;
	case Key_Cent :
		break;
	}
	return retval;
}

void dspLamp(uint8_t lamps) {
	char s[20];

	tfp_sprintf(s, "Lamp : %c %s",
			((lamps == systemData.actLamp) ? '*' : ' '), lampString[lamps]);
	spmc_WriteMenuText(s);
}

uint8_t on_ST_Lamp_Select(int8_t input) {
	static ilum_Type_t ilamps = lt_Bottom;
	static uint8_t retval = ST_Lamp_Select;

	if (menuParameter.enter) {
		menuParameter.enter = 0;
		ilamps = systemData.actLamp;
		dspLamp(ilamps);
		retval = ST_Lamp_Select;
	}

	switch ((uint8_t) input) {
	case Key_LDown:
		if (ilamps == 0) {
			ilamps = lt_Both;
		} else {
			ilamps--;
		}
		dspLamp(ilamps);
		break;
	case Key_Down :
		if (ilamps == 0) {
			ilamps = lt_Both;
		} else {
			ilamps--;
		}
		dspLamp(ilamps);
		break;
	case Key_LUp:
		if (ilamps == 2) {
			ilamps = lt_Bottom;
		} else {
			ilamps++;
		}
		dspLamp(ilamps);
		break;
	case Key_Up :
		if (ilamps == 2) {
			ilamps = lt_Bottom;
		} else {
			ilamps++;
		}
		dspLamp(ilamps);
		break;
	case Key_Cent :
		systemData.actLamp = ilamps;
		dspLamp(ilamps);
		break;
	case Key_Left :
		menuParameter.enter = 1;
		retval = ST_MP_Welcome;
		break;
	case Key_Right :
		menuParameter.enter = 1;
		retval = ST_MP_Time_Select;
		break;
	}
	return retval;
}

void secToArray(uint32_t sec, uint8_t *ta) {
	uint32_t iSec = sec;

	ta[4] = (uint8_t)(iSec / 3600);
	iSec -= (ta[4] * 3600);
	ta[3] = (uint8_t)(iSec / 600);
	iSec -= (ta[3] * 600);
	ta[2] = (uint8_t)(iSec / 60);
	iSec -= (ta[2] * 60);
	ta[1] = (uint8_t)(iSec / 10);
	ta[0] = iSec - (ta[1] * 10);
}

uint32_t getTimeInSec(uint8_t *ta) {
	uint32_t timeInSec;

	timeInSec =  ta[0] * 1;
	timeInSec += ta[1] * 10;
	timeInSec += ta[2] * 60;
	timeInSec += ta[3] * 600;
	timeInSec += ta[4] * 3600;

	return timeInSec;
}

void showTime(uint8_t *at, uint8_t line) {
	char s[20];
	uint32_t presel;

	presel = getTimeInSec(at);

	tfp_sprintf(s, "Time: %c %1d:%1d%1d:%1d%1d",
			((presel == systemData.progSec) ? '*' : ' '),
			at[4], at[3], at[2], at[1], at[0]);
	if (line) {
		spmc_WriteStatusText(s);
	} else {
		spmc_WriteMenuText(s);
	}
}

uint8_t on_ST_Time_Select(int8_t input) {
	static uint8_t retval = ST_Time_Select;
	static uint8_t actTCPos = 0;
	static uint8_t editMode = 0;

	if (menuParameter.enter) {
		menuParameter.enter = 0;
		editMode = FALSE;
		actTCPos = 0;
		showTime(pTime_Digits, 0);
		lcd_gotoxy(0, 1);
		lcd_puts("                ");
		retval = ST_Time_Select;
	}

	switch ((uint8_t) input) {
	case Key_LDown:
		pTime_Digits[4-actTCPos]--;
		if (pTime_Digits[4-actTCPos] > 9) {
			pTime_Digits[4-actTCPos] = 9;
		}
		showTime(pTime_Digits, 0);
		break;
	case Key_Down :
		lcd_gotoxy(0,1);
		lcd_putc(48+actTCPos);
		pTime_Digits[4-actTCPos]--;
		if (pTime_Digits[4-actTCPos] > 9) {
			pTime_Digits[4-actTCPos] = 9;
		}
		showTime(pTime_Digits, 0);
		break;
	case Key_LUp:
		lcd_gotoxy(0,1);
		lcd_putc(48+actTCPos);
		pTime_Digits[4-actTCPos]++;
		if (pTime_Digits[4-actTCPos] > 9) {
			pTime_Digits[4-actTCPos] = 0;
		}
		showTime(pTime_Digits, 0);
		break;
	case Key_Up :
		lcd_gotoxy(0,1);
		lcd_putc(48+actTCPos);
		lcd_gotoxy(0,1);
		lcd_putc(48+actTCPos);
		pTime_Digits[4-actTCPos]++;
		if (pTime_Digits[4-actTCPos] > 9) {
			pTime_Digits[4-actTCPos] = 0;
		}
		showTime(pTime_Digits, 0);
		break;
	case Key_Left :
		if (editMode) {
			actTCPos--;
			if (actTCPos > 5) {
				actTCPos = 4;
			}
			lcd_gotoxy(0, 1);
			lcd_puts("                ");
			lcd_gotoxy(timeCurPos[actTCPos], 1);
			lcd_puts("  ^");
		} else {
			menuParameter.enter = 1;
			retval = ST_MP_Lamp_Select;
		}
		break;
	case Key_Right :
		if (editMode) {
			actTCPos++;
			if (actTCPos > 4) {
				actTCPos = 0;
			}
			lcd_gotoxy(0, 1);
			lcd_puts("                ");
			lcd_gotoxy(timeCurPos[actTCPos], 1);
			lcd_puts("  ^");
		} else {
			menuParameter.enter = 1;
			retval = ST_Init_W_OK;
		}
		break;
	case Key_Cent :
		if (editMode) {
			// save and show
			systemData.progSec = getTimeInSec(pTime_Digits);
			showTime(pTime_Digits, 0);
			lcd_gotoxy(0, 1);
			lcd_puts("                ");
			editMode = FALSE;
		} else {
			editMode = TRUE;
			actTCPos = 0;
			lcd_gotoxy(0, 1);
			lcd_puts("                ");
			lcd_gotoxy(timeCurPos[actTCPos], 1);
			lcd_puts("  ^");
		}
		break;
	}

	return retval;
}

uint8_t on_ST_Init_W_OK(int8_t input){
	static uint8_t retval = ST_Init_W_OK;
	static uint32_t oldSec = 0;

	if (menuParameter.enter) {
		menuParameter.enter = 0;
		spmc_WriteMenuText(MT_InitWithOK);
		showTime(pTime_Digits,1);
		retval = ST_Init_W_OK;
	}

	if (systemData.isCounting) {
		if (oldSec != systemData.actSec) {
			oldSec = systemData.actSec;
			secToArray(systemData.actSec, s_Digits);
			showTime(s_Digits, 1);
			if (systemData.actSec == 0) {
				systemData.isCounting = FALSE;
				menuParameter.enter = 1;
				retval = ST_Init_W_OK;
			}
		}
	}

	switch ((uint8_t) input) {
	case Key_Down :
		menuParameter.enter = 1;
		retval = ST_ResetTimer;
		break;
	case Key_LDown:
		menuParameter.enter = 1;
		retval = ST_ResetTimer;
		break;
	case Key_Up :
		menuParameter.enter = 1;
		retval = ST_ResetTimer;
		break;
	case Key_LUp:
		menuParameter.enter = 1;
		retval = ST_ResetTimer;
		break;
	case Key_Cent :
		if (systemData.isCounting) {
			systemData.isCounting = FALSE;
			spmc_WriteMenuText("... stopped");
		} else {
			if (systemData.actSec) {
				systemData.isCounting = TRUE;
				spmc_WriteMenuText("... counting");
			} else {
				if (systemData.progSec) {
					systemData.actSec = systemData.progSec;
					systemData.isCounting = TRUE;
					spmc_WriteMenuText("... counting");
				}
			}
		}
		break;
	case Key_Left :
		menuParameter.enter = 1;
		retval = ST_MP_Time_Select;
		break;
	}
	return retval;
}

uint8_t on_ST_ResetTimer(int8_t input){
	static uint8_t retval = ST_ResetTimer;

	if (menuParameter.enter) {
		menuParameter.enter = 0;
		spmc_WriteMenuText(MT_ClearCounter);
		retval = ST_ResetTimer;
	}
	switch ((uint8_t) input) {
	case Key_Down :
		menuParameter.enter = 1;
		retval = ST_Init_W_OK;
		break;
	case Key_LDown:
		menuParameter.enter = 1;
		retval = ST_Init_W_OK;
		break;
	case Key_Up :
		menuParameter.enter = 1;
		retval = ST_Init_W_OK;
		break;
	case Key_LUp:
		menuParameter.enter = 1;
		retval = ST_Init_W_OK;
		break;
	case Key_Cent :
		systemData.actSec = 0;
		menuParameter.enter = 1;
		retval = ST_Init_W_OK;
		break;
	}
	return retval;
}


void spmc_WriteMenuText(const char *s) {
//	spmc_WriteToLine(0, s);
	tfp_sprintf(dspLines[0], "%s", s);
	systemData.updateLCD |= (1<<0);
}

void spmc_WriteStatusText(const char *s) {
//	spmc_WriteToLine(3, s);
	tfp_sprintf(dspLines[1], "%s", s);
	systemData.updateLCD |= (1<<1);
}

void spmc_WriteToLine(int line, const char *s) {
	tfp_sprintf(dspLines[line], "%s", s);
	systemData.updateLCD |= (1<<line);
}

void spmc_Update_LCD(void){
	uint8_t lc = 0;

	if (systemData.updateLCD) {
		for (lc = 0; lc < 4; lc++) {
			if (systemData.updateLCD & (1<<lc)) {
				lcd_gotoxy(0,lc);
				lcd_puts("                    ");
				lcd_gotoxy(0,lc);
				lcd_puts(dspLines[lc]);
				systemData.updateLCD &= ~(1<<lc);
			}
		}
	}

	systemData.call_UpdateLCD = FALSE;
}
