/**
 * @file spmc_LCD_Menu.h
 * @author jgnoss
 * @date Sep 23, 2012
 *
 * based on the Menu State machine from AVR Butterfly
 *
 */

#ifndef SPMC_LCD_MENU_H_
#define SPMC_LCD_MENU_H_


//#include <stdlib.h>
#include <inttypes.h>



typedef struct {
	uint8_t state;
	uint8_t input;
	uint8_t nextstate;
} MENU_NEXTSTATE_t;

typedef struct {
	uint8_t state;
	const char *pText;
	uint8_t (*pFunc)(int8_t input);
} MENU_STATE_t;


typedef struct {
	uint8_t lcdM_PrevState;
	uint8_t lcdM_State;
	uint8_t lcdM_NextState;
	uint8_t lcd_SavedState;
	uint8_t enter;
	const char *statetext;
	uint8_t (*pStateFunc)(int8_t);
} menuParameter_t;


extern menuParameter_t menuParameter;


void init_LCDMenu(void);
void handle_LCD_Menu(menuParameter_t *menuParam);

void spmc_Update_LCD(void);
uint8_t lcdMenuStateMachine(uint8_t state, int8_t stimuli);


uint8_t on_ST_Show_Rev(int8_t input);
uint8_t on_ST_EStop(int8_t input);


uint8_t on_ST_Lamp_Select(int8_t input);
uint8_t on_ST_Time_Select(int8_t input);
uint8_t on_ST_Init_W_OK(int8_t input);
uint8_t on_ST_ResetTimer(int8_t input);

void changeToState(uint8_t state, uint8_t saveState);
uint8_t get_lcdSavedState(void);

// Menu state machine states

#define		ST_MP_Welcome			10

#define		ST_Show_Rev				12

#define		ST_MP_SerCLI			13
#define		ST_MP_SerM				14

#define		ST_MP_Run				19

#define		ST_MP_Lamp_Select		20
#define		ST_Lamp_Select			21

#define		ST_MP_Time_Select		30
#define		ST_Time_Select			31

#define		ST_Init_W_OK			40

#define		ST_ResetTimer			50


#define		ST_EStop				249
#define		ST_NotYet				250

extern char dspLines[4][20];

void spmc_WriteMenuText(const char *s);
void spmc_WriteStatusText(const char *s);
void spmc_WriteToLine(int line, const char *s);

#endif /* SPMC_LCD_MENU_H_ */
